/**

 */

#include "node.h"

#include "VuoImageRenderer.h"

VuoModuleMetadata({
                     "title" : "Soft Pixellate Filter",
                     "description" : "Creates a soft pixellation effect, based on https://www.shadertoy.com/view/4dsXWs by Sarconix. Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License. 2014, George Toledo.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoImageRenderer"
					 ],
					 "node": {
						 "isInterface" : false
					 }
				 });

#include "node.h"

static const char * fragmentShaderSource = VUOSHADER_GLSL_SOURCE(120,
	// Inputs
	uniform sampler2D texture;
    uniform vec4 color;
    uniform float pixelSize;
    uniform float ResX;
    uniform float ResY;
    varying vec4 fragmentTextureCoordinate;

    // The min and max sizes of the circles (in pixels) over time.
    //#define MIN_SIZE 2.0
    //#define MAX_SIZE 150.0

    // Compute the relative distance to the circle, where < 0.0 is outside the feathered border,
    // and > 1.0 is inside the feathered border.
    float ComputeCircle(vec2 pos, vec2 center, float radius, float feather)
    {
        // Determine the distance to the center of the circle.
        float dist = length(center - pos);

        // Use the distance and the specified feather factor to determine where the distance lies
        // relative to the circle border.
        float start = radius - feather;
        float end   = radius + feather;
        return smoothstep(start, end, dist);
    }

    // The main function, which is executed once per pixel.
    void main(void)
    {
        vec2 iResolution=vec2(ResX,ResY);
        // Prepare the circle parameters, cycling the circle size over time.
       // float cycle = clamp(.99, 0.0, 1.0);
        float diameter = pixelSize;//mix(MAX_SIZE, MIN_SIZE, cycle);
        // Prepare the circle parameters, cycling the circle size over time.
         // float cycle = clamp(.99, 0.0, 1.0);//getting rid of cycle - george toledo
          float radius = diameter / 2.0;
          vec2  center = vec2(0.0);

          // Compute the relative distance to the circle, using mod() to repeat the circle across the display.
          // A feather value (in pixels) is used to reduce aliasing artifacts when the circles are small.
          // The position is adjusted so that a circle is in the center of the display.
          vec2 screenPos = gl_FragCoord.xy - (iResolution.xy / 2.0) - vec2(radius);
          vec2 pos = mod(screenPos, vec2(diameter)) - vec2(radius);
          float d = ComputeCircle(pos, center, radius, 0.5);

          // Compute "pixelated" (stepped) texture coordinates using the floor() function.
          // The position is adjusted to match the circles, i.e. so a pixelated block is at the center of the
          // display.
          vec2 count = iResolution.xy / diameter;
          vec2 shift = vec2(0.5) - fract(count / 2.0);
          vec2 uv = floor(count * gl_FragCoord.xy / iResolution.xy + shift) / count;

          // Sample the texture, using an offset to the center of the pixelated block.
          // NOTE: Use a large negative bias to effectively disable mipmapping, which would otherwise lead
          // to sampling artifacts where the UVs change abruptly at the pixelated block boundaries.
          uv += vec2(0.5) / count;
          uv = clamp(uv, 0.0, 0.99);
          //uv.y = 1.0 - uv.y;
          vec4 texColor = texture2D(texture, uv, -32.0);//change to control background color.-gt

          // Calculate the color based on the circle shape, mixing between that color and a background color.
          // NOTE: Set the mix factor to 0.0 to see the pixelating effect directly, without the circles.
          vec4 bg  = color;//change to control background color.-gt
          vec4 col = mix(texColor, bg, d);//change to control background color.-gt

          // Set the final fragment color.
          gl_FragColor = vec4(col);
    }

);
struct nodeInstanceData
{
	VuoShader shader;
	VuoGlContext glContext;
	VuoImageRenderer imageRenderer;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData * instance = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));
	VuoRegister(instance, free);

	instance->glContext = VuoGlContext_use();

    instance->shader = VuoShader_make("Soft Pixellate Shader");
	VuoShader_addSource(instance->shader, VuoMesh_IndividualTriangles, NULL, NULL, fragmentShaderSource);
	VuoRetain(instance->shader);

	instance->imageRenderer = VuoImageRenderer_make(instance->glContext);
	VuoRetain(instance->imageRenderer);

	return instance;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) instance,
		VuoInputData(VuoImage) image,

        VuoInputData(VuoReal, {"default":5.0,"suggestedStep":0.01}) pixelSize,
        VuoInputData(VuoColor, {"default":{"r":0.25,"g":0.25,"b":0.25,"a":0.25}}) color,


		VuoOutputData(VuoImage) adjustedImage
)
{
	if (! image)
		return;

	// Feed parameters to the shader.
	VuoShader_setUniform_VuoImage((*instance)->shader, "texture", image);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "pixelSize", pixelSize);
    VuoShader_setUniform_VuoColor ((*instance)->shader, "color", color);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResX",   image->pixelsWide);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResY",   image->pixelsHigh);
;

	// Render.
	*adjustedImage = VuoImageRenderer_draw((*instance)->imageRenderer, (*instance)->shader, image->pixelsWide, image->pixelsHigh, VuoImage_getColorDepth(image));
}

void nodeInstanceFini(VuoInstanceData(struct nodeInstanceData *) instance)
{
	VuoRelease((*instance)->shader);
	VuoRelease((*instance)->imageRenderer);
	VuoGlContext_disuse((*instance)->glContext);
}
